// Copyright 2022 Intel Corporation
// SPDX-License-Identifier: Apache-2.0

#pragma once

// ospray
#include "OSPCommon.h"

namespace ospray {

struct RendererData
{
  virtual ~RendererData() = default;
};

} // namespace ospray
