// Copyright 2009 Intel Corporation
// SPDX-License-Identifier: Apache-2.0

#pragma once

#include "rkcommon/os/library.h"

namespace ospray {

using rkcommon::getSymbol;
using rkcommon::loadLibrary;

} // namespace ospray
