// Copyright 2009 Intel Corporation
// SPDX-License-Identifier: Apache-2.0

// \file ospray/ospray.h Defines the external OSPRay API */

// NOTE: this is a single include for all the C++ wrappers around ospray.h

#include "ospray_cpp/Camera.h"
#include "ospray_cpp/Data.h"
#include "ospray_cpp/Device.h"
#include "ospray_cpp/FrameBuffer.h"
#include "ospray_cpp/Future.h"
#include "ospray_cpp/GeometricModel.h"
#include "ospray_cpp/Group.h"
#include "ospray_cpp/ImageOperation.h"
#include "ospray_cpp/Instance.h"
#include "ospray_cpp/Light.h"
#include "ospray_cpp/Material.h"
#include "ospray_cpp/Renderer.h"
#include "ospray_cpp/Texture.h"
#include "ospray_cpp/TransferFunction.h"
#include "ospray_cpp/VolumetricModel.h"
#include "ospray_cpp/World.h"
